@@ -1,6 +1,6 @@
 import numpy as np
 
-from ..itertools import flatten_seq
+from ..core.itertools import flatten_seq
 
 __all__ = [
     'isconst',
@@ -74,7 +74,7 @@ def meshgrid(*arrs, **kwargs):
         
     arrs = (arr[...,None] for arr in arrs)
     arrs = meshgrid_dd(*arrs)
-    arrs = [arr.squeeze() for arr in arrs]
+    arrs = [arr.squeeze(-1) for arr in arrs]
     
     if indexing == 'xy':
         arrs = [np.swapaxes(arr, 0, 1) for arr in arrs]
